/* =============================================================================================================
	SyncthingIndicator 0.1
================================================================================================================

	GJS Syncthing gnome-shell panel indicator signalling the Syncthing deamon status.

	Credits to <jay.strict@posteo.de> for the reference implementation

	Copyright (c) 2016, 2nv2u <info@2nv2u.com>
	This work is distributed under GPLv3, see LICENSE for more information.
============================================================================================================= */

const Lang = imports.lang;
const Gio = imports.gi.Gio;
const Gtk = imports.gi.Gtk;
const St = imports.gi.St;

const Main = imports.ui.main;
const Animation = imports.ui.animation;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const StatusSystem = imports.ui.status.system;

const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();

const Logger = Me.imports.logger;
const console = new Logger.Service(Logger.Level.DEBUG,'SYNCTHING-UI'); 

const Syncthing = Me.imports.syncthing;

const SyncthingPanelIcon = new Lang.Class({
	Name: 'SyncthingPanelIcon',

	_init: function() {
		let scaleFactor = St.ThemeContext.get_for_stage(global.stage).scale_factor;
		this._workingIcon = new Animation.Animation(
			Gio.File.new_for_path(Me.path+'/icons/syncthing-working.svg'),
			20,
			20,
			80
		);
		this._idleIcon = St.TextureCache.get_default().load_file_async(
			Gio.File.new_for_path(Me.path+'/icons/syncthing-idle.svg'),
			20,
			20,
			scaleFactor
		);
		this._pausedIcon = St.TextureCache.get_default().load_file_async(
			Gio.File.new_for_path(Me.path+'/icons/syncthing-paused.svg'),
			20,
			20,
			scaleFactor
		);
		this._disconnectedIcon = St.TextureCache.get_default().load_file_async(
			Gio.File.new_for_path(Me.path+'/icons/syncthing-disconnected.svg'),
			20,
			20,
			scaleFactor
		);
		this.actor = new St.Bin();
		this.actor.set_child(this._disconnectedIcon);
	},

	setState: function(state){
		this._workingIcon.stop();
		switch(state){
			case Syncthing.State.SYNCING:
			case Syncthing.State.SCANNING:
				this.actor.set_child(this._workingIcon.actor);
				this._workingIcon.play();
			break
			case Syncthing.DeviceState.PAUSED:
				this.actor.set_child(this._pausedIcon);
			break;
			case Syncthing.State.UNKNOWN:
			case Syncthing.DeviceState.DISCONNECTED:
				this.actor.set_child(this._disconnectedIcon);
			break;
			default:
				this.actor.set_child(this._idleIcon);
			break
		}
	}
});

const RescanMenuItem = new Lang.Class({
	Name: 'RescanMenuItem',
	Extends: PopupMenu.PopupBaseMenuItem,

	_init: function () {
		this.parent();
		this.setSensitive(false);
		this.actor.add_child(
			new St.Icon({
				gicon: new Gio.ThemedIcon({ name: 'emblem-synchronizing-symbolic' }),
				style_class: 'popup-menu-icon'
			})
		);
		
		this._label = new St.Label({ text: 'Rescan All Folders' });
		this.actor.add_child(this._label);
		this.actor.label_actor = this._label;
		
		this.actor.add_style_class_name('syncthing-menu-spacing');
	},

	activate: function(event) {
		syncthingManager.rescan();
	}
});

const ConfigMenuItem = new Lang.Class({
	Name: 'ConfigMenuItem',
	Extends: PopupMenu.PopupBaseMenuItem,

	_init: function(){
		this.parent();
		this.setSensitive(false);

		this.actor.add_child(
			new St.Icon({
				gicon: new Gio.ThemedIcon({ name: 'preferences-system-symbolic' }),
				style_class: 'popup-menu-icon'
			})
		);
		this.actor.add_child(
			new St.Label({ text: 'Open Web Interface' })
		);

		this.actor.add_style_class_name('syncthing-menu-spacing');
	},

	activate: function(event) {
		let launchContext = global.create_app_launch_context(event.get_time(), -1);
		try {
			Gio.AppInfo.launch_default_for_uri(syncthingManager.getURI(), launchContext);
		} catch(e) {
			Main.notifyError(_("Failed to launch URI \"%s\"").format(uri), e.message);
		}
	}
});

const SectionMenu = new Lang.Class({
	Name: 'SectionMenu',
	Extends: PopupMenu.PopupSubMenuMenuItem,

	_init: function(title,icon){
		this.parent(title,true);
		this.icon.icon_name = icon;
		this.icon.add_style_class_name('syncthing-submenu-icon');
		this.section = new PopupMenu.PopupMenuSection();
		this.section.actor.add_style_class_name('syncthing-menu-spacing');
		this.menu.addMenuItem(this.section);
	}

});

const FolderMenu = new Lang.Class({
	Name: 'FolderMenu',
	Extends: SectionMenu,

	_init: function(){
		this.parent('Folders','system-file-manager-symbolic');

		this.rescan = new RescanMenuItem();
		this.menu.addMenuItem(this.rescan);

		syncthingManager.connect(Syncthing.Signal.SERVICE_CHANGE, Lang.bind(this,function(manager,service){
			switch(service.state){
				case Syncthing.ServiceState.ENABLED:
					if(service.name == 'syncthing'){
						this.rescan.setSensitive(true);
					}
				break;
				case Syncthing.ServiceState.DISABLED:
					if(service.name == 'syncthing'){
						this.rescan.setSensitive(false);
					}
				break;
			}
		}));
	}
});

const FolderMenuItem = new Lang.Class({
	Name: 'FolderMenuItem',
	Extends: PopupMenu.PopupBaseMenuItem,
	
	_init: function(folder) {
		this.parent();
		this._folder = folder;
		
		let icon;
		let file = Gio.File.new_for_path(this._folder.path);
		try {
			icon = file.query_info('standard::symbolic-icon', 0, null).get_symbolic_icon();
		} catch(e if e instanceof Gio.IOErrorEnum){
			if(!file.is_native()){
				icon = new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
			} else {
				icon = new Gio.ThemedIcon({ name: 'folder-symbolic' });
			}
		}
		
		this.icon = new St.Icon({ gicon: icon, style_class: 'popup-menu-icon' });
		this.actor.add_child(this.icon);
		
		this.label = new St.Label({ text: folder.name });
		this.actor.add_child(this.label);
		this.actor.label_actor = this.label;

		this.path = file.get_uri();
		
		this._folder.connect(Syncthing.Signal.STATE_CHANGE, Lang.bind(this,function(folder,state){
			this.icon.style_class = 'popup-menu-icon '+state;
		}));

		this._folder.connect(Syncthing.Signal.REMOVE, Lang.bind(this,function(folder){
			this.destroy();
		}));
		
	},

	activate: function(event) {
		try {
			let launchContext = global.create_app_launch_context(event.get_time(), -1);
			Gio.AppInfo.launch_default_for_uri(this.path, launchContext);
		} catch(e) {
			Main.notifyError(_("Failed to launch URI \"%s\"").format(this._uri), e.message);
		}
		this.parent(event);
	}
});

const DeviceMenu = new Lang.Class({
	Name: 'DeviceMenu',
	Extends: SectionMenu,

	_init: function(){
		this.parent('This device','computer-symbolic');

		this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());

		this._switches = {
			'syncthing': new PopupMenu.PopupSwitchMenuItem("Syncthing Service", false, null),
			'inotify': new PopupMenu.PopupSwitchMenuItem("Inotify Service", false, null)
		};

		for(let service in this._switches){
			this._switches[service].connect('activate', function(_service){
				return function(actor,event){
					if(actor.state) {
						syncthingManager.enableService(_service);
					} else {
						syncthingManager.disableService(_service);
					}
				}
			}(service));
			this.menu.addMenuItem(this._switches[service]);
		}

		this.config = new ConfigMenuItem();
		this.menu.addMenuItem(this.config);

		syncthingManager.connect(Syncthing.Signal.SERVICE_CHANGE, Lang.bind(this,function(manager,service){
			switch(service.state){
				case Syncthing.ServiceState.ENABLED:
					this._switches[service.name].setToggleState(true);
					if(service.name == 'syncthing'){
						this.config.setSensitive(true);
					}
				break;
				case Syncthing.ServiceState.DISABLED:
					this._switches[service.name].setToggleState(false);
					if(service.name == 'syncthing'){
						this.config.setSensitive(false);
					}
				break;
			}
		}));
	},

	setDevice: function(device){
		this.device = device;
		this.label.text = device.name;
		this.device.connect(Syncthing.Signal.STATE_CHANGE, Lang.bind(this,function(device,state){
			this.icon.style_class = 'popup-menu-icon syncthing-submenu-icon '+state;
		}));
	}
});

const DeviceMenuItem = new Lang.Class({
	Name: 'DeviceMenuItem',
	Extends: PopupMenu.PopupSwitchMenuItem,
	
	_init: function(device) {
		this.parent(device.name, false, null);

		this._device = device;

		let icon = new Gio.ThemedIcon({ name: 'network-server-symbolic' });
		this.icon = new St.Icon({ gicon: icon, style_class: 'popup-menu-icon' });
		this.actor.insert_child_at_index(this.icon,1);
		
		this.setSensitive(false);

		this._device.connect(Syncthing.Signal.STATE_CHANGE, Lang.bind(this,function(device){
			let state = device.getState()
			switch(state){
				case Syncthing.DeviceState.DISCONNECTED:
					this.setSensitive(false);
					this.setToggleState(false);
				break;
				case Syncthing.DeviceState.PAUSED:
					this.setSensitive(true);
					this.setToggleState(false);
				break;
				default:
					this.setSensitive(true);
					this.setToggleState(true);
				break
			}
			this.icon.style_class = 'popup-menu-icon '+state;
		}));

		this._device.connect(Syncthing.Signal.REMOVE, Lang.bind(this,function(){
			this.destroy();
		}));

		this.connect('activate', Lang.bind(this,function(actor){
			if(actor.state) {
				this._device.resume();
			} else {
				this._device.pause();
			}
		}));
	}
});

const SyncthingIndicator = new Lang.Class({
	Name: 'SyncthingIndicator',
	Extends: PanelMenu.Button,

	_init: function() {
		this.parent(0.0, "SyncthingIndicator");

		this.icon = new SyncthingPanelIcon();
		this.actor.add_actor(this.icon.actor);
		
		this.deviceMenu = new DeviceMenu();
		this.menu.addMenuItem(this.deviceMenu);

		this.folderMenu = new FolderMenu();
		this.menu.addMenuItem(this.folderMenu);

		this.defaultMenu = this.deviceMenu;

		this.menu.connect('open-state-changed', Lang.bind(this, function(menu,state){
			if(state) this.defaultMenu.menu.open();
		}));
		
		syncthingManager.connect(Syncthing.Signal.SERVICE_CHANGE, Lang.bind(this,function(manager,service){
			switch(service.state){
				case Syncthing.ServiceState.ENABLED:
					if(service.name == 'syncthing'){
						this.defaultMenu = this.folderMenu;
					}
				break;
				case Syncthing.ServiceState.DISABLED:
					if(service.name == 'syncthing'){
						this.defaultMenu = this.deviceMenu;
					}
				break;
			}
		}));
		
		syncthingManager.connect(Syncthing.Signal.FOLDER_ADD, Lang.bind(this,function(manager,folder){
			this.folderMenu.setSensitive(true);	
			this.folderMenu.section.addMenuItem(
				new FolderMenuItem(folder)
			);
		}));

		syncthingManager.connect(Syncthing.Signal.DEVICE_ADD, Lang.bind(this,function(manager,device){
			this.deviceMenu.section.addMenuItem(
				new DeviceMenuItem(device)
			);
		}));

		syncthingManager.connect(Syncthing.Signal.HOST_ADD, Lang.bind(this,function(manager,device){
			this.deviceMenu.setDevice(device);
			device.connect(Syncthing.Signal.STATE_CHANGE, Lang.bind(this,function(device,state){
				this.icon.setState(state);
			}));
		}));
	}
});

function init(extension) {
	Gtk.IconTheme.get_default().append_search_path(extension.path + '/icons');
}

let syncthingManager;
let syncthingIndicator;

function enable() {
	syncthingManager = new Syncthing.Manager();
	syncthingIndicator = new SyncthingIndicator();
	Main.panel.addToStatusArea('syncthingIndicator', syncthingIndicator);
	syncthingManager.attach();
}

function disable() {
	syncthingIndicator.destroy();
	syncthingManager.destroy();
}
